/*=================================================================================================
NOTE:
	This file contains the command processing for the programming of NAND flash. 
	PLEASE DO NOT make any change!
==================================================================================================*/


#include "Types.h"
#include "Comm.h"
#include "Flash.h"


//COMMAND DEFINITIONS
#define NAND_READ_ID				0
#define NAND_READ_PAGE				1
#define NAND_PROGRAM_PAGE			2
#define NAND_ERASE_BLOCK			3
#define NAND_CHECK_BLANK			4
#define NAND_IS_BADBLOCK			5
#define NAND_INFO_TABLE				6



/*=================================================================================================
FUNCTION: read_id()          

DESCRIPTION: 
	Used to read flash ID.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void read_id(void)
{
	U32 id;	
		
	//Read Flash ID
	id = nand_read_id();

	//Return Flash ID
	write_to_host(id);
}



/*=================================================================================================
FUNCTION: read_page()          

DESCRIPTION: 
	Used to read one page.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void read_page(void)
{
	U32 pageidx;	
	U32 blockidx;
		
	//Read block idx
	blockidx = read_from_host();

	//Read page idx
	pageidx = read_from_host();
	
	//Read one page
	nand_read_page(blockidx, pageidx);
}



/*=================================================================================================
FUNCTION: program_page()          

DESCRIPTION: 
	Used to program one page.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void program_page(void)
{
	U32 ret;
	U32 pageidx;	
	U32 blockidx;
		
	//Read block idx
	blockidx = read_from_host();

	//Read page idx
	pageidx = read_from_host();
	
	//Program one page
	ret = nand_program_page(blockidx, pageidx);
	
	//Return status
	write_to_host(ret);	
}



/*=================================================================================================
FUNCTION: erase_block()          

DESCRIPTION: 
	Used to erase one block.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void erase_block(void)
{
	U32 ret;
	U32 blockidx;
		
	//Read block idx
	blockidx = read_from_host();
	
	//Erase one block
	ret = nand_erase_block(blockidx);
	
	//Return status
	write_to_host(ret);	
}



/*=================================================================================================
FUNCTION: check_blank()          

DESCRIPTION: 
	Used to check if one block is blank or not.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void check_blank(void)
{
	U32 ret;
	U32 blockidx;
		
	//Read block idx
	blockidx = read_from_host();
	
	//Check one block
	ret = nand_check_blank(blockidx);
	
	//Return status
	write_to_host(ret);	
}



/*=================================================================================================
FUNCTION: is_badblock()          

DESCRIPTION: 
	Used to check if one block is bad/invalid.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void is_badblock(void)
{
	U32 ret;
	U32 blockidx;
		
	//Read block idx
	blockidx = read_from_host();
	
	//Check one block
	ret = nand_is_badblock(blockidx);
	
	//Return status
	write_to_host(ret);	
}



/*=================================================================================================
FUNCTION: receive_table()          

DESCRIPTION: 
	Used to receive the bad block table and relocation table..
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
void info_table(void)
{
	U32 ret;
	
	//Receive Table
	ret = nand_info_table();
	
	//Send Ack
	write_to_host(ret);	
}



/*=================================================================================================
FUNCTION: main_func()          

DESCRIPTION: 
	Command processing main function. Never return.
   
ARGUMENTS PASSED:
	None
			
RETURN VALUE:
	None
==================================================================================================*/
int main_func()
{
	U32 cmd;	

	//Init Flash
	nand_init();
	
	while(1)
	{
		cmd = read_from_host();

		switch(cmd)
		{	
		case NAND_READ_ID:				//Read ID
			read_id();
			break;
		case NAND_READ_PAGE:			//Read One Page
			read_page();
			break;			
		case NAND_PROGRAM_PAGE:			//Program One Page
			program_page();
			break;
		case NAND_ERASE_BLOCK:			//Erase One Block
			erase_block();
			break;
		case NAND_CHECK_BLANK:			//Check Blank
			check_blank();
			break;
		case NAND_IS_BADBLOCK:			//Is Bad/Invalid Block?
			is_badblock();		
			break;
		case NAND_INFO_TABLE:			//Receive Table
			info_table();
			break;						
		default:	
			break;
		}

	}
	
	return 0;
}



